﻿using C1.Win.C1TrueDBGrid;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.OleDb;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ReadCellStyle
{
    public partial class Form1 : Form
    {
        private List<string> _colors;
        private static int counter = 0;

        public Form1()
        {
            InitializeComponent();

            _colors = new List<string>
            {
                "LightCyan",
                "Beige",
                "LightBlue",
                "LightGray"
            };

            Load += Form1_Load;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            _trueDB.DataSource = GetDataTable("InventoryManagement");

            foreach (C1DisplayColumn column in _trueDB.Splits[0].DisplayColumns)
                column.FetchStyle = true;

            _trueDB.FetchCellStyle += _trueDB_FetchCellStyle;
            button1.Click += Button1_Click;
        }

        private void Button1_Click(object sender, EventArgs e)
        {
            int row = -1, col = -1;
            if (textBox1.Text == string.Empty)
                return;

            string[] rowCol = textBox1.Text.Split(' ');
            if (rowCol.Count() != 2)
                return;
            if (!int.TryParse(rowCol[0], out row) || !int.TryParse(rowCol[1], out col))
                return;

            if (row < 0 && row >= _trueDB.Rows.Count)
                MessageBox.Show($"Row value must be between 0 and {_trueDB.Rows.Count - 1}");

            if (col < 0 && col >= _trueDB.Columns.Count)
                MessageBox.Show($"Column value must be between 0 and {_trueDB.Columns.Count - 1}");

            var method = _trueDB.Splits[0].GetType().GetMethods(System.Reflection.BindingFlags.NonPublic |
                System.Reflection.BindingFlags.Instance).Where(m => m.Name.Contains("GetCellStyle")).
                First();

            var style = method.Invoke(_trueDB.Splits[0], new object[] { row, col, _trueDB.Columns[col].CellText(row), false });
            
            MessageBox.Show($"Row: {row}, Col: {col}, BackColor: {(style as Style)?.BackColor}");
        }

        private void _trueDB_FetchCellStyle(object sender, FetchCellStyleEventArgs e)
        {
            ++counter;
            int index = _trueDB.Columns.Count * e.Row + e.Col;
            e.CellStyle.BackColor = Color.FromName(_colors[index % _colors.Count]);
        }

        private DataTable GetDataTable(string tableName)
        {
            string connectionString = GetConnectionString();
            string commandText = $"select * from [{tableName}]";
            OleDbDataAdapter adapter = new OleDbDataAdapter(commandText, connectionString);
            DataTable table = new DataTable(tableName);
            adapter.Fill(table);

            return table;
        }

        private string GetConnectionString()
        {
            string path = Environment.GetFolderPath(Environment.SpecialFolder.Personal) + @"\ComponentOne Samples\Common";
            string conn = "Provider=Microsoft.Jet.OLEDB.4.0; Data Source={0}\\C1NWind.mdb";
            return String.Format(conn, path);
        }
    }
}
